;
$DATE (07.03.2006)
$TITLE (TmTest.a51)
$PAGELENGTH(56)
$PAGEWIDTH(150)
$DEBUG
$XREF
$NOLIST
$NOMOD51
$INCLUDE(89S8252.MCU)
$LIST
;
;********************************************************************************************************
; Testprogramm fr Temperatursensor LM 75
; J. Rathlev, Mrz 2006
;********************************************************************************************************

      NAME      TMTest

;********************************************************************************************************
; Konstanten
; ==========

CODEMEM EQU     8000H

LED     EQU     P1.0    ; Portanschluss fr LED
TAKT    EQU     P1.7

; Berechnung des Zhlerwertes:
;     N = 65536 - T/s * f/MHz / 12  

; f(Takt) = 24 MHz: 10 ms ==> 45536

; f(Takt) = 22,1184 MHz: 10 ms ==> 47104
 
 
TIMCNT  EQU     47104   ; Zhlerwert fr 10 ms bei 22,1184 MHz

TIntvl  EQU     100      ; Anzahl der 10ms Takte fr 1000 ms
                
;********************************************************************************************************
; RAM-Daten
; =========

        DSEG    AT      30H

TmSAdr: DS      1               ; Subadresse des Portexpanders
TmDat:  DS      2   
TCount: DS      1

        BSEG      
        
TmFlag: DBIT    1               ; Flag fr neuen Temperaturwert

;********************************************************************************************************
; ext. Datenspeicher
; ==================

        XSEG    AT 0C000H

;********************************************************************************************************
; Programmstart

        CSEG    AT      CODEMEM

        LJMP    Start

        ORG     CODEMEM+ExtI0

        RETI
        
        ORG     CODEMEM+Timer0

        LJMP    T0IRQ       
        
;********************************************************************************************************

        ORG     CODEMEM+30H

$INCLUDE(i2c.a51)
$INCLUDE(SerInOut.a51)

;********************************************************************************************************
; Timer 0 Interrupt

USING   2                       ; Registerbank 2 verwenden

T0IRQ:  PUSH    PSW
        MOV     PSW,#010H       ; Registerbank 2 einschalten

        MOV     TH0,#HIGH(TIMCNT)
        MOV     TL0,#LOW(TIMCNT)
        CPL     TAKT
        DJNZ    TCOUNT,T0RET
        CPL     LED             ; LED an/aus

        MOV     R0,#TmDat
        MOV     R1,#2
        MOV     R2,TmSAdr
        CALL    TmRead
        
        SETB    TmFlag          ; neuer Wert
        MOV     TCOUNT,#TIntvl

T0RET:  POP     PSW             ; Flags wiederherstellen 
        RETI
   
USING   0                       ; Registerbank 0 verwenden


;********************************************************************************************************
; Initialisieren

InitTmSens:
        MOV     TmSAdr,A
        CLR     TmFlag
        RET        


;********************************************************************************************************
; Timer 0 initialisieren, Interrupts freischalten

InitTimer:
        MOV     A,TMOD      
        ANL     A,#0F0H         ; Ausmaskieren von Timer 0 (Bit 0..3)
        ORL     A,#01H          ; Setze Mode 1, int. Zhltakt
        MOV     TMOD,A
        MOV     TH0,#HIGH(TIMCNT) ; Anfangswert fr 10 ms
        MOV     TL0,#LOW(TIMCNT)
        CLR     TF0
        MOV     TCOUNT,#TIntvl  ; Zhler fr Zeitintervall
        SETB    TR0             ; Starte Timer
        SETB    ET0             ; Enable Timer-Interrupt
        RET

;********************************************************************************************************
; Hauptprogramm

Start:  MOV     SP,#STACKSTART
        MOV     A,#244          ; BR-Count (Serint.a51)
        CALL    InitSerial
        MOV     A,#0            ; Subadresse
        CALL    InitTmSens
        CALL    InitTimer
        SETB    EA              ; Enable Global-Interrupt

LOOP:   JNB     TmFlag,LOOP     ; warte
        CLR     TmFlag
        
        MOV     DPTR,#TEXT1
        CALL    WriteFixStr
        
        MOV     A,TmDat
        cpl     A
        mov     P1,A
        MOV     A,TmDat
        MOV     B,A
        JNB     ACC.7,Lp1       ; positiv
        CPL     A
        INC     A               ; 2er-Komplement
        MOV     B,A
        MOV     A,#'-'
        CALL    WriteChar       ; Vorzeichen
        SJMP    Lp2
        
Lp1:    MOV     A,#32           
        CALL    WriteChar       ; Leerzeichen
Lp2:    MOV     A,B
        CALL    WriteDec 
        MOV     A,#','
        CALL    WriteChar       ; Komma 
        MOV     A,TmDat+1       ; Nachkommastelle
        JB      ACC.7,Lp3
        MOV     A,#'0'
        SJMP    Lp4
        
Lp3:    MOV     A,#'5'
Lp4:    CALL    WriteChar       ; letzte Stelle
           
        MOV     DPTR,#TEXT2
        CALL    WriteFixStr
        CALL    WriteCrLf
        SJMP    LOOP
        
Text1:  DB      'T = ',0
Text2:  DB      ' ',186,'C',0

;********************************************************************************************************
        DSEG

STACKSTART:
        DS      1       ; stack: rest of data memory

END
