;$NOLIST

; Include Datei

;********************************************************************************************************
; Abfrage einer Zehnertastatur mit zwei Sondertasten (12-pol. Anschluss)
; ber einen I2C-Portexpander
; J. Rathlev, Aug. 2004

; Erforderlich:  Include-Datei "I2C.a51"

; Interrupt: Int0 bei Tastenbettigung
; Anschlsse:
; Wert    Tastatur       E-Port 0      E-Port 1
;   1        9                           0
;   2        5             4
;   3        1             0
;   4       10                           1
;   5        6             5
;   6        2             1
;   7       11                           2 
;   8        7             6
;   9        3             2
;   *       12                           3
;   0        8             7 
;   #        4             3
;           13   - Gnd

;********************************************************************************************************
        DSEG

; RAM-Daten
; =========

TaSAdr: DS      1               ; Subadresse des Portexpanders 
TaDat:  DS      2
                                                                                     
        BSEG

TaFlag:  DBIT    1               ; Flag fr erfolgte Tastatureingabe


;********************************************************************************************************
        CSEG

; Tastatur-Interrupt
; ==================

SVPC    SET     $

        ORG     CODEMEM+ExtI0

        LJMP    EX0IRQ

        ORG     SVPC
                
; Interrupt-Routine

USING   2                       ; Registerbank 2 verwenden

EX0IRQ: PUSH    PSW
        MOV     PSW,#010H       ; Registerbank 2 einschalten

        MOV     R0,#TaDat
        MOV     R1,#1
        MOV     R2,TaSAdr
        CALL    PE0Read
        MOV     R1,#1
        INC     R2
        CALL    PE0Read
        SETB    TaFlag

X0RET:  POP     PSW             ; Flags wiederherstellen 
        RETI


USING   0                       ; Registerbank 0 verwenden


;********************************************************************************************************
; Initialisieren
; Im Hauptprogramm muss noch folgen:
;         SETB    EA              ; Enable Global-Interrupt
; Subadresse des Portexpanders in A

InitKey:
        MOV     TaSAdr,A
        CLR     TaFlag
        SETB    EX0             ; Int0 von Tastatur
        RET        

;********************************************************************************************************
; Zuordnung der Tasten zu den ASCII-Codes

TTable: DB '0852#963*741'

; Prfen, ob Eingabe erfolgt ist, und Umsetzen der Eingabe in ASCII
; Benutzt:      A,R7,DPTR
; Ergebnis:     C = 1 : Eingabe erfolgt
;               A = Eingabewert
;               C = 0 : keine Eingabe


GetKey: JNB     TaFlag,GkRet    ; keine Eingabe erfolgt
        CLR     TaFlag
        MOV     A,TaDat         ; low byte
        MOV     R7,#8
GkLp1:  RRC     A
        JNC     GkLp4           ; Bit = 0 ?
        DJNZ    R7,GkLp1
        MOV     A,TaDat+1       ; high byte
        MOV     R7,#4     
GkLp2:  RRC     A
        JNC     GkLp3
        DJNZ    R7,GkLp2
        SJMP    GkRet           ; keine Taste gedrckt ???

GkLp3:  MOV     A,R7
        ADD     A,#8
        MOV     R7,A
GkLp4:  MOV     A,R7
        DEC     A
        MOV     DPTR,#TTable
        MOVC    A,@A+DPTR

        SETB    C
        RET

GkRet:  CLR      C
        RET

$LIST
